#ifndef STATE_MACHINE_H
#define STATE_MACHINE_H

#include "Events.h"
#include "StateChanger.h"
#include "StateFactory.h"
#include "StateContext.h"

class State;

//------------------------------------------------------------------------------
//
class StateMachine : public Events, StateChanger, StateFactory
//
/// @brief Our state machine.
///
/// We are derived from Events so that the outside world can inform us of the
/// events we care about.
///
/// We are derived from StateChanger so that we know how to change State, and
/// can be used by the State objects, without them knowing about this class.
///
/// We are derived from StateFactory so that our State objects can get
/// new state objects without caring about the mechanics of allocation. It is
/// easier to derive than to have a member, as the factory needs a lot of
/// context to create the states, and we'd just have to pass it all in.
///
//------------------------------------------------------------------------------
{
public:

    StateMachine (Actions &);

    ~StateMachine();

    // Implement Events interface
    void Dark ();
    void Light ();
    void Movement ();
    void NoMovement ();
    void Timeout ();

    // Implement StateChanger interface
    void ChangeState (State &newState) { currentState = &newState; }

    // Implement StateFactory interface
    State & DayState () { return *day; }
    State & OffState () { return *off; }
    State & MovingState () { return *moving; }
    State & TimingState () { return *timing; }

    // Get the current state, in an abstract way.
    const char * Report () const;

private:

    Actions &actions;

    StateContext stateContext;

    // States for the StateFactory to dish out.
    State *day;
    State *off;
    State *moving;
    State *timing;

    State *currentState;

    // for lint.
    StateMachine (const StateMachine &);
    StateMachine & operator= (const StateMachine &);
};

#endif // STATE_MACHINE_H
